function out = cluster1d(n, Nsamples, nu)

% Carries out the calculations for Section 4 of the paper. Enter this 
% command to duplicate the examples there:
% out = cluster1d(10, 1000, .5);

close all
rng(123); % to set the random seed
dimension = 1;
c = nu^dimension;
Nplot = 1000; % number of points to plot 

out = cluster1drun(2);
out1 = out;
out = cluster1drun(3);
out2 = out;
out = cluster1drun(4);
out3 = out;
out = {out1, out2, out3};

%%%%%%%%%%%%%%%%%%%%%%%%% subfunction 1 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function out = cluster1drun(p)

        %Set the classically I-optimal design points
        tstar = zeros(p,1);
        if p==2
            tstar = [-1;1];
        elseif p==3
            tstar = [-1; 0; 1];
        elseif p==4
            tstar = [-1; -.4472; .4472; 1];
        end

        % Form the intervls of support J_i(c) of the beta densities
        s = [min(-1, tstar(1)); .5*(tstar(1:(p-1)) + tstar(2:p)); max(1, tstar(p))];
        I = [s(1:p), s(2:(p+1))];
        J = [tstar - c*(tstar - s(1:p)), tstar + c*(s(2:(p+1)) - tstar)];
        k = J(:,1);
        l = J(:,2);

        % Get the parameters of Beta densities
        a = ones(p,1)/c;
        b = ones(p,1)/c;
        m = (tstar - k)./(l - k);
        for i=1:p
            if (i < (p+1)/2)
                a(i) = 1 + (b(i) - 1)*m(i)/(1 - m(i));
            else
                b(i) = 1 + (a(i) - 1)*(1 - m(i))/m(i);
            end
        end
        params = [a b];

        % Gather samples

        weights = (I(:,2) - I(:,1))/2;
        subsamplesizes = round(n*weights);
        if (rem(p,2) == 1) && sum(subsamplesizes)> n
            subsamplesizes((p+1)/2) = subsamplesizes((p+1)/2) - 1;
        end

        Samples = zeros(n, Nsamples);
        for count = 1:Nsamples
            sample = [];
            for i= 1:p
                newsample = (l(i)-k(i))*betarnd(a(i), b(i), subsamplesizes(i), 1)+ k(i);
                sample = [sample; newsample];
            end
            Samples(:, count) = sample;
        end
      
        % Compute M_PHI
        intM = zeros(p,1);
        for j = 1:p
            fun = @(x) (x.^(2*(j-1))).*phi(x);
            intM(j) = integral(@(x)fun(x), -1,1);
        end
        M = zeros(p,p);
        for i = 1:p
            for j = 1:p
                if rem(i+j,2)==0
                    M(i,j) = intM((i+j)/2);
                end
            end
        end

        % Compute K_PHI
        intK = zeros(p,1);
        for j = 1:p
            fun = @(x) (x.^(2*(j-1))).*phi(x).*phi(x);
            intK(j) = integral(@(x)fun(x), -1,1);
        end
        K = zeros(p,p);
        for i = 1:p
            for j = 1:p
                if rem(i+j,2)==0
                    K(i,j) = intK((i+j)/2);
                end
            end
        end


        % Compute A
        intA = zeros(p,1);
        for j = 1:p
            fun = @(x) (x.^(2*(j-1)));
            intA(j) = integral(@(x)fun(x), -1,1);
        end
        A = zeros(p,p);
        for i = 1:p
            for j = 1:p
                if rem(i+j,2)==0
                    A(i,j) = intA((i+j)/2);
                end
            end
        end
 
        %Compute H and G and rootG
        H = M'*(A\M);
        G = K-H;
        G = (G + G')/2;
        [V,D] = eig(G);
        rootG = V*real(sqrt(D))*V';

        % Compute beta
        P = eye(p) + rootG'*(H\rootG);
        [V,D] = eig(P);
        [~,ind] = sort(diag(D));
        %Ds = D(ind,ind);
        Vs = V(:,ind);
        beta = Vs(:,p);
        betastar = rootG\beta; % G^(-1/2) times beta
        mats = {A, M, H, K, G, rootG};

            
        % Compute max loss
        variance = trace(M\A);
        maxbias = max(eig(H\K));
        maxloss = (1-nu)*variance + nu*maxbias;  

        % Estimate J(PHI)
        Jphi = zeros(Nsamples,1);
        for i = 1:Nsamples
            x = Samples(:,i);
            F = zeros(n,p);
            for j = 1:p
                F(:,j) = x.^(j-1);
            end
            Dphi = diag(phi(x'));
            Mdelta = F'*F/n;
            Mphi = F'*Dphi*F/n;
            rightvec = (Mdelta\Mphi - A\M)*betastar;
            Jphi(i) = (1-nu)*trace(Mdelta\A) + nu*(1+rightvec'*A*rightvec);
        end
        JPHI = mean(Jphi);


        % plotting
        figure (1)
        subplot(2,3,(p-1))
        hold on
        xlim([-1.01 1.01]);
        X = linspace(-1,1,Nplot)';
        plot(X,phi(X'))
        xticks(tstar)
        if p==2
            xticklabels({'t_1', 't_2'});
            xlabel({'(a): p = 2, \phi(x);'  ; ['subsample sizes \{', num2str(subsamplesizes'), '\}']});
        elseif p==3
            xticklabels({'t_1', 't_2', 't_3'});
            xlabel({'(b): p = 3, \phi(x);'  ; ['subsample sizes \{', num2str(subsamplesizes'), '\}']});
        elseif p==4
            xticklabels({'t_1', 't_2', 't_3', 't_4'});
            xlabel({'(c): p = 4, \phi(x);'  ; ['subsample sizes \{', num2str(subsamplesizes'), '\}']});
        end
        plot(Samples(:,1), .01, 'k.')
        hold off

        subplot(2,3,3+(p-1))
        hold on
        ylim([-2,2]);
        F = ones(p, Nplot);
        for j = 2:p
            F(j,:) = X'.^(j-1);
        end
        psiPhi = betastar'*(F.*phi(X') - M*(A\F));
        plot(X, psiPhi);
        if p==2
            xlabel('(d): p = 2, \psi_\Phi(x)');
        elseif p==3
            xlabel('(e): p = 3, \psi_\Phi(x)');
        elseif p==4
            xlabel('(f): p = 4, \psi_\Phi(x)');
        end
        hold off

        figure(2)
        subplot(1,3,(p-1))
        hold on
        if p==2
            xlabel('(a): p = 2');
        elseif p==3
            xlabel('(b): p = 3');
        elseif p==4
            xlabel('(c): p = 4');
        end

        histogram(Jphi, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
        text1 = (['average j_{\nu}(\delta) = ', num2str(round(JPHI,2))]);
        plot(JPHI, 0,  'k*', 'DisplayName', text1);
        text2 = (['I_{\nu}(\Phi) = ', num2str(round(maxloss,2))]);
        plot(maxloss, 0,  'g*', 'DisplayName', text2);
        legend ('Location', 'northeast')
        hold off


        performance = [variance, maxbias, maxloss, JPHI];
        colnames = {'variance', 'maxbias', 'maxloss', 'aver[IMSE]'};
        performance = array2table(performance,'VariableNames',colnames);
        out =  {performance, weights, sort(sample), mats, beta};


        %%%%%%%%%%%%%%%%%%%%%%%%% subfunction 2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        function out = phi(x)
            x = x';
            y = zeros(size(x,1),p);
            for s = 1:p
                y(:,s) = (k(s)<=x).*(x<=l(s)).*betapdf((x- k(s))./(l(s) - k(s)),...
                    params(s,1), params(s,2))./(2*c);
            end
            out = (y*ones(p,1))';
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       
        end
    end
end