function [misc] = cluster2d(loaddata, nu, n, tile, alpha, Nsamples, twosamplesizes)

% Carries out the computations for Section 5 of the paper.
% To duplicate what is there use 'loaddata = 1'; this 
% avoids having to do all the integrations again.
% With loaddata = 0; it will re-compute and store
% PHI and the integrals, for use next time

% alpha determines the axial points
% alpha = 1 for a face centre cube design;
%       = sqrt(2) for a spherical designs;
%       = sqrt(2)/2 for an inscribed design
% 'tile' is the tile to be highlighted


% EXAMPLE:
% [misc] = cluster2d(1, .5, 50, 8, sqrt(2), 1000, 1) % As in the paper


% misc (miscellany) = [radii, weights, subsamplesizes, numrejects, ...
% estimatedintegrals, integrals, phiweights];

 close all  
rng(606); % to set the random seed
k = 2; %dimensionality
p0 = 6; % number of regressors
a = k; % first parameter of beta density
c = nu^k; % 'c' is the factor by which the tiles are are uniformly ...
% scaled to get the J(i). e.g. c = .25 gives subtiles J one quarter ...
% the area of the containing tiles
b = 1/c; % b is the second parameter in the beta densities used in sampling
N = 20; % N^2 points are plotted within each tile.
    
if (loaddata)
    load('mydata', 'base', 'PHI', 'misc', 'lambdas', 'kappas', 'A', 'M', 'K', 'Sample', 'Rejections')
end

if (alpha == sqrt(2))
    l = 2;
elseif (alpha == sqrt(2)/2)
    l = 1;
elseif (alpha == 1)
    l = sqrt(2);
end

xlimit = [-l l];
ylimit = [-l l];
xbox = xlimit([1 1 2 2 1]);
ybox = ylimit([1 2 2 1 1]);

% Form a tessellation generated by tstar:
axialpoints = [alpha 0; 0 alpha; -alpha 0; 0 -alpha];
corners = [-1 1; -1 -1; 1 -1; 1 1];
tstar1 = [corners; axialpoints; 0 0];
extras = 4*corners; % added so that the regions I need are bounded
tstar = [tstar1; extras];

dt = delaunayTriangulation(tstar);
[V,R]= voronoiDiagram(dt);
tstar = tstar1;
q = size(tstar,1);

figure(1)
subplot(1,2,1)
hold on
xlabel('(a)')
ll = 2*l;
xlim([-ll ll]);
ylim([-ll ll]);
voronoi(dt)
for i = 1:size(V,1)
    %text(V(i,1), V(i,2), num2str(i))
    plot(V(i,1), V(i,2),'*')
end
for i = 1:q
    text(tstar(i,1), tstar(i,2), ['t_{',num2str(i),'}'])
    plot(tstar(i,1), tstar(i,2),'*')
end
line(xbox,ybox, 'color', 'black');
hold off

poly1 = polyshape(xbox, ybox);
Areas = zeros(q,2);
Tiles = cell(q);
J = Tiles;

for i = 1:q
    poly2 = polyshape([V(R{i},1),V(R{i},2)]);
    Tiles{i} = intersect(poly1,poly2);
    Areas(i,1) = area(Tiles{i});
    J{i} = scale(Tiles{i},sqrt(c),[tstar(i,1), tstar(i,2)]);
    Areas(i,2) = area(J{i});
end

% Max distance from tstar(i) to boundary of J{i}:
Radii = zeros(q,1);
for i = 1:q
    p = tstar(i,:);
    [a1,a2] = boundary(J{i});
    nvertices = size(a1,1) - 1;
    jvertices = [a1 a2];
    jvertices = jvertices(1:nvertices,:);
    span = zeros(nvertices,1);
    for j = 1:nvertices
        span(j) = norm(p - jvertices(j,:));
    end
    Radii(i) = max(span);
end


subplot(1,2,2)
hold on
xlabel('(b)')
xlim([-l l]);
ylim([-l l]);
voronoi(dt);
for i = 1:q
    plot(J{i})
    text(tstar(i,1), tstar(i,2), ['t_{',num2str(i),'}'])
    plot(tstar(:,1), tstar(:,2),'.');
end
viscircles(tstar, Radii);
line(xbox,ybox, 'color', 'black')
hold off

% Sampling preliminaries
weights = Areas(:,1)/sum(Areas(:,1));
subsamplesizes = round(n*weights);
subsamplesizes(9) = n - sum(subsamplesizes(1:8));
estimatedintegrals = zeros(q,1);
numrejects = zeros(9,1);

if ~(loaddata)
    integrals = zeros(q,1);
    for i = 1:q
        r = Radii(i);
        xmin = tstar(i,1) - r;
        xmax = tstar(i,1) + r;
        ymin = @(x1) tstar(i,2)-sqrt(r^2 - (x1-tstar(i,1)).^2);
        ymax =  @(x1) tstar(i,2)+sqrt(r^2 - (x1-tstar(i,1)).^2);

        fun = @(x1,x2,i) f(x1,x2,i);
        integrals(i) = integral2(@(x1,x2)fun(x1,x2,i), xmin, xmax, ymin, ymax);
        t1 = linspace(tstar(i,1) - r , tstar(i,1) + r, N);
        t2 = linspace(tstar(i,2) - r , tstar(i,2) + r, N);
        px1 = ones(N,1)*t1;
        px2 = t2'*ones(1,N);
        phi{i} = {px1; px2; (weights(i)/integrals(i))*f(px1,px2,i)};
    end

    xmat = [];
    ymat = xmat;
    zmat = xmat;
    for i = 1:q
        xmat = [xmat; phi{i}{1}];
        ymat = [ymat; phi{i}{2}];
        zmat = [zmat; phi{i}{3}];
    end

    PHI = {xmat; ymat; zmat};
    phiweights = weights./integrals;

    basemin = -2;
    basemax = 2;
    nz = 6*N; % nz^2 zeros will be plotted first
    base1 = linspace(basemin,basemax, nz);
    base2 = base1;
    bx1 = ones(nz,1)*base1;
    bx2 = base2'*ones(1,nz);
    base = {bx1; bx2; zeros(size(bx1))};

    lambda0 = zeros(q,1);
    lambda2 = zeros(q,1);
    lambda22 = zeros(q,1);
    lambda4 = zeros(q,1);

    kappa0 = zeros(q,1);
    kappa2 = zeros(q,1);
    kappa22 = zeros(q,1);
    kappa4 = zeros(q,1);

    for i = 1:q
        r = Radii(i);
        xmin = tstar(i,1) - r;
        xmax = tstar(i,1) + r;
        ymin = @(x1) tstar(i,2)-sqrt(r^2 - (x1-tstar(i,1)).^2);
        ymax =  @(x1) tstar(i,2)+sqrt(r^2 - (x1-tstar(i,1)).^2);

        lambda0fun = @(x1,x2,i) f(x1,x2,i);
        lambda2fun = @(x1,x2,i) (x1.^2).*f(x1,x2,i);
        lambda22fun = @(x1,x2,i) (x1.^2).*(x2.^2).*f(x1,x2,i);
        lambda4fun = @(x1,x2,i) (x1.^4).*f(x1,x2,i);

        kappa0fun = @(x1,x2,i) f(x1,x2,i).^2;
        kappa2fun = @(x1,x2,i) (x1.^2).*(f(x1,x2,i).^2);
        kappa22fun = @(x1,x2,i) (x1.^2).*(x2.^2).*(f(x1,x2,i).^2);
        kappa4fun = @(x1,x2,i) (x1.^4).*(f(x1,x2,i).^2);

        lambda0(i) = integral2(@(x1,x2)lambda0fun(x1,x2,i), xmin, xmax, ymin, ymax);
        lambda2(i) = integral2(@(x1,x2)lambda2fun(x1,x2,i), xmin, xmax, ymin, ymax);
        lambda22(i) = integral2(@(x1,x2)lambda22fun(x1,x2,i), xmin, xmax, ymin, ymax);
        lambda4(i) =  integral2(@(x1,x2)lambda4fun(x1,x2,i), xmin, xmax, ymin, ymax);

        kappa0(i) = integral2(@(x1,x2)kappa0fun(x1,x2,i), xmin, xmax, ymin, ymax);
        kappa2(i) = integral2(@(x1,x2)kappa2fun(x1,x2,i), xmin, xmax, ymin, ymax);
        kappa22(i) = integral2(@(x1,x2)kappa22fun(x1,x2,i), xmin, xmax, ymin, ymax);
        kappa4(i) =  integral2(@(x1,x2)kappa4fun(x1,x2,i), xmin, xmax, ymin, ymax);
    end

    lambda0 = sum(phiweights.*lambda0);
    lambda2 = sum(phiweights.*lambda2);
    lambda22 = sum(phiweights.*lambda22);
    lambda4 = sum(phiweights.*lambda4);

    kappa0 = sum(phiweights.*phiweights.*kappa0);
    kappa2 = sum(phiweights.*phiweights.*kappa2);
    kappa22 = sum(phiweights.*phiweights.*kappa22);
    kappa4 = sum(phiweights.*phiweights.*kappa4);

    lambdas = [lambda0, lambda2, lambda22, lambda4];
    kappas = [kappa0, kappa2, kappa22, kappa4];

    A = [180 0 0 0 15 15; 0 15 0 0 0 0;   0 0 15 0 0 0; ...
        0 0 0 20 0 0; 15 0 0 0 36 20; 15 0 0 0 20 36];
    A = (64/45)*A;

    M = [1 0 0 0 lambda2 lambda2; 0 lambda2 0 0 0 0;               0 0 lambda2 0 0 0; ...
        0 0 0 lambda22 0 0;      lambda2 0 0 0 lambda4 lambda22; lambda2 0 0 0 lambda22 lambda4];

    K = [kappa0 0 0 0 kappa2 kappa2; 0 kappa2 0 0 0 0;            0 0 kappa2 0 0 0; ...
        0 0 0 kappa22 0 0;     kappa2 0 0 0 kappa4 kappa22; kappa2 0 0 0 kappa22 kappa4];


    %Take one sample for the plots
    Sample = {};
    Rejections = {};
    for i = 1:q
        ni = subsamplesizes(i);
        acceptedsample = [];
        rejections = [];
        while  (size(acceptedsample,1) < ni)
            rho = betarnd(a, b, 1, 1);
            theta = unifrnd(0, 2*pi);
            r = Radii(i);
            newx = tstar(i,:) + r*rho*[cos(theta), sin(theta)]; % 1 by 2
            if (isinterior(J{i}, newx))
                acceptedsample = [acceptedsample; newx];
            else
                rejections = [rejections; newx];
            end
        end
        Sample{i} = acceptedsample;
        Rejections{i} = rejections;
        estimatedintegrals(i) = ni/(ni + size(rejections,1));
        numrejects(i) = size(rejections,1);
    end

    miscellany = [Radii, Areas, subsamplesizes, numrejects, ...
        estimatedintegrals, integrals, weights, phiweights];
    colNames = {'Radii', 'TileAreas', 'SubTileAreas', 'n_i', ...
        'numrejects', 'estimatedintegrals', 'integrals', 'weights', 'phiweights'};
    misc = array2table(miscellany,'VariableNames',colNames);
    save('mydata', 'base', 'PHI', 'misc', 'lambdas', 'kappas', 'A', 'M', 'K', 'Sample', 'Rejections')

end

figure(3)
subplot(1,2,1)
hold on
sample = [];
lim = l;
xlim([-lim,lim]);
ylim([-lim,lim]);
for i = 1:9
    sample = [sample;Sample{i}];
end
scatter(sample(:,1),sample(:,2));
scatter(tstar(:,1),tstar(:,2), 'k+')
line(xbox, ybox,'Color', 'black');
xlabel('(a)');
hold off

subplot(1,2,2)
hold on
plot(Tiles{tile}, 'FaceColor','cyan')
plot(J{tile},  'FaceColor','yellow')
plot(tstar(tile,1), tstar(tile,2), 'r+');
viscircles(tstar(tile,:),Radii(tile));
sample = Sample{tile};
rejects = Rejections{tile};
scatter(sample(:,1), sample(:,2), 'bo');
if (size(rejects,1) > 0)
    scatter(rejects(:,1), rejects(:,2), 'kx');
end
xlabel('(b)');
hold off

figure(4)
hold on
surf(base{1}, base{2}, base{3},'EdgeColor','blue')
%surf(PHI{1}, PHI{2}, PHI{3},'EdgeColor','yellow')
%mesh(base{1}, base{2}, base{3},'FaceAlpha', '0')
mesh(PHI{1}, PHI{2}, PHI{3},'FaceAlpha', '1')
xlabel('x_{1}');
ylabel('x_{2}');
zlabel('\phi(\bfx; \nu)');
mycolors = [0 0 0; 1 0 0; 0 0 1];% black; red; blue
colormap(mycolors);
%colormap(autumn(5))
view(3);
hold off

%% Compute I(PHI)
%Compute H and G and rootG
H = M'*(A\M);
G = K-H;
G = (G + G')/2;
[V,D] = eig(G);
rootG = V*real(sqrt(D))*V';

% Compute max loss
variance = trace(M\A)
maxbias = max(eig(H\K))
maxloss = (1-nu)*variance + nu*maxbias % This is I(PHI)
Iphi = maxloss;

% Compute beta
P = eye(p0) + rootG'*(H\rootG);
[V,D] = eig(P);
[~,ind] = sort(diag(D));
%Ds = D(ind,ind);
Vs = V(:,ind);
beta0 = Vs(:,p0);
betastar = rootG\beta0; % G^(-1/2) times beta

%5 Take many stratified samples; compute Mdelta and Mphi and evaluate
% j(delta) for each


if ~(twosamplesizes)
    figure(5)
    [jdelta, Jhat] = sampler(n);
    hold on
    histogram(jdelta, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
    text1 = (['average j_{\nu}(\delta) = ', num2str(round(Jhat))]);
    plot(Jhat, 0,  'k*', 'DisplayName', text1);
    text2 = (['I_{\nu}(\Phi) = ', num2str(round(Iphi))]);
    plot(Iphi, 0,  'g*', 'DisplayName', text2);
    legend ('Location', 'northeast')
    hold off

elseif (twosamplesizes)
    figure(5)
    [jdelta, Jhat] = sampler(n);
    subplot(1,2,1)
    hold on
    histogram(jdelta, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
    text1 = (['average j_{\nu}(\delta) = ', num2str(round(Jhat))]);
    plot(Jhat, 0,  'k*', 'DisplayName', text1);
    text2 = (['I_{\nu}(\Phi) = ', num2str(round(Iphi))]);
    plot(Iphi, 0,  'g*', 'DisplayName', text2);
    legend ('Location', 'northeast')
    xlabel('(a): n=50');
    hold off

    [jdelta, Jhat] = sampler(2*n);
    subplot(1,2,2)
    hold on
    histogram(jdelta, 'HandleVisibility', 'off', 'FaceColor', 'yellow' )
    text1 = (['average j_{\nu}(\delta) = ', num2str(round(Jhat))]);
    plot(Jhat, 0,  'k*', 'DisplayName', text1);
    text2 = (['I_{\nu}(\Phi) = ', num2str(round(Iphi))]);
    plot(Iphi, 0,  'g*', 'DisplayName', text2);
    legend ('Location', 'northeast')
    xlabel('(b): n=100');
    hold off

end
%%%%%%%%%%%%%%%%%%%%%%%%% subfunctions %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function out = f(x1,x2,i)
    s1 = size(x1,1);
    s2 = size(x1,2);
    X1 = x1 - tstar(i,1);
    X2 = x2 - tstar(i,2);
    norms = sqrt(X1.^2 + X2.^2);
    radius = Radii(i);
    scalednorms = norms/radius;
    normalizer = 2*(pi^(k/2))*(radius^k)*beta(a,b)/gamma(k/2);
    out = (scalednorms.^(a-k)).*((1-scalednorms).^(b-1))/normalizer;
    Pin = [reshape(x1,s1*s2,1), reshape(x2,s1*s2,1)];
    Pout = (isinterior(J{i},Pin));
    P = reshape(Pout,s1,s2);
    out = out.*P;
end

function [jdelta, Jhat] = sampler(n2sample)
    n = n2sample;
    subsamplesizes = round(n*weights);
    subsamplesizes(9) = n - sum(subsamplesizes(1:8));
    jdelta = zeros(Nsamples, 1);
    for jj = 1:Nsamples
        sampledata = [];
        for i = 1:q
            r = Radii(i);
            ni = subsamplesizes(i);
            acceptedsample = [];
            while  (size(acceptedsample,1) < ni)
                rho = betarnd(a, b, 1, 1);
                theta = unifrnd(0, 2*pi);
                newx = tstar(i,:) + r*rho*[cos(theta), sin(theta)]; % 1 by 2
                if (isinterior(J{i}, newx))
                    acceptedsample = [acceptedsample; newx];
                end
            end
            newdata = zeros(ni,3);
            phiweights = misc.phiweights;
            phi_i = @(x1,x2,i) phiweights(i)*f(x1,x2,i);
            for j = 1:ni
                newdata(j,:) = [acceptedsample(j,:), phi_i(acceptedsample(j,1),acceptedsample(j,2),i)];
            end
            sampledata = [sampledata; newdata];
        end
        % form Mdelta and Mphi
        X1 = sampledata(:,1);
        X2 = sampledata(:,2);
        F = [ones(n,1) X1 X2 X1.*X2 X1.^2 X2.^2];
        Mdelta = F'*F/n;
        Mphi = F'*diag(sampledata(:,3))*F/n;
        % Compute j(delta)
        v0 = (Mdelta\Mphi - A\M)*betastar;
        gammadelta = v0'*A*v0 + 1;
        jdelta(jj) = (1-nu)*trace(Mdelta\A) + nu*gammadelta;
    end
    Jhat = mean(jdelta)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

end


